@@ -63,6 +63,7 @@
 #include "gpm-brightness.h"
 #include "gpm-tray-icon.h"
 #include "gpm-inhibit.h"
+#include "gpm-polkit.h"
 #include "gpm-stock-icons.h"
 #include "gpm-manager.h"
 
@@ -102,12 +103,14 @@ struct GpmManagerPrivate
 
 	GpmDpms		*dpms;
 	GpmIdle		*idle;
+	GpmHal		*hal;
 	GpmInfo		*info;
 	GpmFeedback	*feedback;
 	GpmPower	*power;
 	GpmBrightness   *brightness;
 	GpmScreensaver  *screensaver;
 	GpmInhibit	*inhibit;
+	GpmPolkit	*polkit;
 
 	guint32          ac_throttle_id;
 	guint32          dpms_throttle_id;
@@ -121,11 +124,11 @@ struct GpmManagerPrivate
 	GpmWarning	 last_keyboard_warning;
 	GpmWarning	 last_pda_warning;
 
-	gint		 last_primary_percentage_change;
-
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
 	gboolean	 done_notify_fully_charged;
+	gboolean	 enable_beeping;
+	gboolean	 ignore_inhibits;
 
 	time_t		 last_resume_event;
 	int		 suppress_policy_timeout;
@@ -193,9 +196,13 @@ static gboolean
 gpm_manager_is_policy_timout_valid (GpmManager *manager,
 				    const char *action)
 {
+	char *message;
 	if ((time (NULL) - manager->priv->last_resume_event) <=
 	    manager->priv->suppress_policy_timeout) {
-		gpm_debug ("Skipping suppressed %s", action);
+		message = g_strdup_printf ("Skipping suppressed %s", action);
+		gpm_info_event_log (manager->priv->info,
+				    GPM_GRAPH_EVENT_NOTIFICATION, message);
+		g_free (message);
 		return FALSE;
 	}
 	return TRUE;
@@ -221,7 +228,11 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 
 	action_ok = gpm_inhibit_check (manager->priv->inhibit);
 	if (! action_ok) {
-		title = g_strdup_printf ("Request to %s", action);
+		if (manager->priv->ignore_inhibits) {
+			gpm_debug ("Inhibit ignored through gconf policy!");
+			return TRUE;
+		}
+		title = g_strdup_printf (_("Request to %s"), action);
 		GString *message = g_string_new ("");
 		gpm_inhibit_get_message (manager->priv->inhibit, message, action);
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
@@ -230,6 +241,9 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 				      GPM_NOTIFY_TIMEOUT_LONG,
 				      GTK_STOCK_DIALOG_WARNING,
 				      GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		g_string_free (message, TRUE);
 		g_free (title);
 	}
@@ -249,13 +263,19 @@ gpm_manager_allowed_suspend (GpmManager *manager,
 			     gboolean   *can,
 			     GError    **error)
 {
-	gboolean gconf_policy;
+	gboolean gconf_ok;
+	gboolean polkit_ok = TRUE;
+	gboolean hal_ok = FALSE;
 	g_return_val_if_fail (can, FALSE);
 
 	*can = FALSE;
-	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
 					      GPM_PREF_CAN_SUSPEND, NULL);
-	if ( gconf_policy && gpm_hal_can_suspend () ) {
+	hal_ok = gpm_hal_can_suspend (manager->priv->hal);
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-suspend");
+#endif
+	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
 
@@ -275,16 +295,21 @@ gpm_manager_allowed_hibernate (GpmManager *manager,
 			       gboolean   *can,
 			       GError    **error)
 {
-	gboolean gconf_policy;
+	gboolean gconf_ok;
+	gboolean polkit_ok = TRUE;
+	gboolean hal_ok = FALSE;
 	g_return_val_if_fail (can, FALSE);
 
 	*can = FALSE;
-	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
 					      GPM_PREF_CAN_HIBERNATE, NULL);
-	if ( gconf_policy && gpm_hal_can_hibernate () ) {
+	hal_ok = gpm_hal_can_hibernate (manager->priv->hal);
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-hibernate");
+#endif
+	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
-
 	return TRUE;
 }
 
@@ -300,11 +325,12 @@ gpm_manager_allowed_shutdown (GpmManager *manager,
 			      gboolean   *can,
 			      GError    **error)
 {
-	if (can) {
-		*can = FALSE;
-	}
-	/* FIXME: check other stuff */
-	if (can) {
+	*can = FALSE;
+	gboolean polkit_ok = TRUE;
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-shutdown");
+#endif
+	if ( polkit_ok ) {
 		*can = TRUE;
 	}
 	return TRUE;
@@ -322,11 +348,12 @@ gpm_manager_allowed_reboot (GpmManager *manager,
 			    gboolean   *can,
 			    GError    **error)
 {
-	if (can) {
-		*can = FALSE;
-	}
-	/* FIXME: check other stuff */
-	if (can) {
+	*can = FALSE;
+	gboolean polkit_ok = TRUE;
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-reboot");
+#endif
+	if ( polkit_ok ) {
 		*can = TRUE;
 	}
 	return TRUE;
@@ -640,7 +667,7 @@ change_power_policy (GpmManager *manager,
 	gpm_brightness_set_level_std (manager->priv->brightness, brightness);
 	gpm_brightness_set (manager->priv->brightness);
 
-	gpm_hal_enable_power_save (power_save);
+	gpm_hal_enable_power_save (manager->priv->hal, power_save);
 	update_ac_throttle (manager, on_ac);
 
 	/* set the new sleep (inactivity) value */
@@ -841,27 +868,62 @@ manager_policy_do (GpmManager *manager,
 /**
  * gpm_manager_get_on_ac:
  * @manager: This manager class instance
- * @on_ac: TRUE if we are on AC power
+ * @retval: TRUE if we are on AC power
  **/
 gboolean
 gpm_manager_get_on_ac (GpmManager  *manager,
-			gboolean   *on_ac,
+			gboolean   *retval,
 			GError    **error)
 {
+	gboolean on_ac;
+
 	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
 
-	if (on_ac) {
-		gpm_power_get_on_ac (manager->priv->power, on_ac, error);
+	if (retval == NULL) {
+		return FALSE;
 	}
 
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, error);
+	*retval = on_ac;
+
 	return TRUE;
 }
 
+/**
+ * gpm_manager_get_low_power_mode:
+ * @manager: This manager class instance
+ * @retval: TRUE if we are on low power mode
+ **/
+gboolean
+gpm_manager_get_low_power_mode (GpmManager  *manager,
+				gboolean    *retval,
+				GError     **error)
+{
+	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
+
+	if (retval == NULL) {
+		return FALSE;
+	}
+
+	gboolean on_ac;
+	gboolean power_save;
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, error);
+	if (on_ac) {
+		power_save = gconf_client_get_bool (manager->priv->gconf_client,
+						    GPM_PREF_AC_LOWPOWER, NULL);
+	} else {
+		power_save = gconf_client_get_bool (manager->priv->gconf_client,
+						    GPM_PREF_BATTERY_LOWPOWER, NULL);
+	}
+	*retval = power_save;
+
+	return TRUE;
+}
 /**
  * gpm_manager_set_dpms_mode:
  * @manager: This manager class instance
  * @mode: The DPMS mode, e.g. GPM_DPMS_MODE_STANDBY
- * Return value: TRUE if we could set the GPMS mode OK.
+ * Return value: TRUE if we could set the DPMS mode OK.
  **/
 gboolean
 gpm_manager_set_dpms_mode (GpmManager *manager,
@@ -979,7 +1041,7 @@ gpm_manager_shutdown (GpmManager *manager,
 				   GNOME_SAVE_GLOBAL,
 				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
 
-	gpm_hal_shutdown ();
+	gpm_hal_shutdown (manager->priv->hal);
 	ret = TRUE;
 
 	return ret;
@@ -1012,7 +1074,7 @@ gpm_manager_reboot (GpmManager *manager,
 				   GNOME_SAVE_GLOBAL,
 				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
 
-	gpm_hal_reboot ();
+	gpm_hal_reboot (manager->priv->hal);
 	ret = TRUE;
 
 	return ret;
@@ -1055,7 +1117,7 @@ gpm_manager_hibernate (GpmManager *manager,
 
 	gpm_networkmanager_sleep ();
 
-	ret = gpm_hal_hibernate ();
+	ret = gpm_hal_hibernate (manager->priv->hal);
 	manager_explain_reason (manager, GPM_GRAPH_EVENT_RESUME,
 				_("Resuming computer"), NULL);
 
@@ -1145,7 +1207,7 @@ gpm_manager_suspend (GpmManager *manager,
 	int charge_before_suspend = status.current_charge;
 
 	/* Do the suspend */
-	ret = gpm_hal_suspend (0);
+	ret = gpm_hal_suspend (manager->priv->hal, 0);
 	manager_explain_reason (manager, GPM_GRAPH_EVENT_RESUME,
 				_("Resuming computer"), NULL);
 
@@ -1496,12 +1558,12 @@ lid_button_pressed (GpmManager	 *manager,
 	if (state) {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_LID_CLOSED,
-				    "The laptop lid has been closed");
+				    _("The laptop lid has been closed"));
 		gpm_debug ("lid button CLOSED");
 	} else {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_LID_OPENED,
-				    "The laptop lid has been re-opened");
+				    _("The laptop lid has been re-opened"));
 		gpm_debug ("lid button OPENED");
 	}
 
@@ -1641,11 +1703,11 @@ power_on_ac_changed_cb (GpmPower   *power,
 	if (on_ac) {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_ON_AC,
-				    "AC adapter inserted");
+				    _("AC adapter inserted"));
 	} else {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_ON_BATTERY,
-				    "AC adapter removed");
+				    _("AC adapter removed"));
 	}
 
 	/* We do the lid close on battery action if the ac_adapter is removed
@@ -1786,7 +1848,7 @@ manager_critical_action_do (GpmManager *manager)
  * and also do the critical actions here.
  **/
 static void
-battery_status_changed_primary (GpmManager	      *manager,
+battery_status_changed_primary (GpmManager     *manager,
 				GpmPowerKind    battery_kind,
 				GpmPowerStatus *battery_status)
 {
@@ -1806,12 +1868,13 @@ battery_status_changed_primary (GpmManager	      *manager,
 		manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	}
 
-	/* We have to track the last percentage, as when we do the transition
-	 * 99 to 100 some laptops report this as charging, some as not-charging.
-	 * This is probably a race. This method should work for both cases. */
-	if (manager->priv->last_primary_percentage_change == 99 &&
-	    battery_status->percentage_charge == 100 &&
-	    ! manager->priv->done_notify_fully_charged) {
+	/* We use the hardware charged state instead of the old 99%->100%
+	 * percentage charge method, as the icon must disappear when this
+	 * notification is shown (if we set to the display policy "charging")
+	 * and also the fact that some ACPI batteries don't make it to 100% */
+	if (manager->priv->done_notify_fully_charged == FALSE && 
+	    gpm_power_battery_is_charged (battery_status)) {
+
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_BATTCHARGED, NULL);
 		if (show_notify) {
@@ -1832,12 +1895,10 @@ battery_status_changed_primary (GpmManager	      *manager,
 	/* We only re-enable the fully charged notification when the battery
 	   drops down to 95% as some batteries charge to 100% and then fluctuate
 	   from ~98% to 100%. See #338281 for details */
-	if (manager->priv->last_primary_percentage_change < 95) {
+	if (battery_status->percentage_charge < 95) {
 		manager->priv->done_notify_fully_charged = FALSE;
 	}
 
-	manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
-
 	if (! battery_status->is_discharging) {
 		gpm_debug ("%s is not discharging", gpm_power_kind_to_localised_string (battery_kind));
 		return;
@@ -1933,6 +1994,9 @@ battery_status_changed_primary (GpmManager	      *manager,
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      title, message, timeout,
 				      icon, GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_NOTIFICATION,
 				    title);
@@ -2040,6 +2104,9 @@ battery_status_changed_ups (GpmManager	   *manager,
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      title, message, GPM_NOTIFY_TIMEOUT_LONG,
 				      icon, GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_NOTIFICATION,
 				    title);
@@ -2123,6 +2190,9 @@ battery_status_changed_misc (GpmManager	    	   *manager,
 			      GPM_NOTIFY_TIMEOUT_LONG,
 			      icon,
 			      GPM_NOTIFY_URGENCY_NORMAL);
+	if (manager->priv->enable_beeping) {
+		gpm_warning_beep ();
+	}
 	gpm_info_event_log (manager->priv->info, GPM_GRAPH_EVENT_NOTIFICATION, title);
 
 	g_free (message);
@@ -2310,6 +2380,15 @@ gconf_key_changed_cb (GConfClient *client,
 						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
 		gpm_brightness_set_level_dim (manager->priv->brightness, brightness);
 
+	} else if (strcmp (entry->key, GPM_PREF_ENABLE_BEEPING) == 0) {
+		manager->priv->enable_beeping =
+			gconf_client_get_bool (manager->priv->gconf_client,
+					       GPM_PREF_ENABLE_BEEPING, NULL);
+
+	} else if (strcmp (entry->key, GPM_PREF_IGNORE_INHIBITS) == 0) {
+		manager->priv->ignore_inhibits =
+			gconf_client_get_bool (manager->priv->gconf_client,
+					       GPM_PREF_IGNORE_INHIBITS, NULL);
 	}
 }
 
@@ -2358,7 +2437,7 @@ gpm_manager_tray_icon_suspend (GpmManager   *manager,
 		return;
 	}
 	manager_explain_reason (manager,
-				GPM_GRAPH_EVENT_HIBERNATE,
+				GPM_GRAPH_EVENT_SUSPEND,
 				_("Suspending computer"),
 				_("user clicked suspend from tray menu"));
 	gpm_manager_suspend (manager, NULL);
@@ -2445,6 +2524,17 @@ screensaver_connection_changed_cb (GpmScreensaver *screensaver,
 	}
 }
 
+/**
+ * hal_daemon_monitor_cb:
+ * @hal: The HAL class instance
+ **/
+static void
+hal_daemon_monitor_cb (GpmHal     *hal,
+		     GpmManager *manager)
+{
+	tray_icon_update (manager);
+}
+
 /**
  * gpm_manager_init:
  * @manager: This manager class instance
@@ -2470,6 +2560,13 @@ gpm_manager_init (GpmManager *manager)
 	g_signal_connect (manager->priv->power, "battery-status-changed",
 			  G_CALLBACK (power_battery_status_changed_cb), manager);
 
+	manager->priv->hal = gpm_hal_new ();
+	g_signal_connect (manager->priv->hal, "daemon-start",
+			  G_CALLBACK (hal_daemon_monitor_cb), manager);
+	g_signal_connect (manager->priv->hal, "daemon-stop",
+			  G_CALLBACK (hal_daemon_monitor_cb), manager);
+	
+
 	manager->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (manager->priv->screensaver, "connection-changed",
 			  G_CALLBACK (screensaver_connection_changed_cb), manager);
@@ -2516,6 +2613,9 @@ gpm_manager_init (GpmManager *manager)
 	/* use a class to handle the complex stuff */
 	manager->priv->inhibit = gpm_inhibit_new ();
 
+#ifdef HAVE_POLKIT
+	manager->priv->polkit = gpm_polkit_new ();
+#endif
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
 	g_signal_connect (G_OBJECT (manager->priv->tray_icon), "destroy",
@@ -2523,8 +2623,6 @@ gpm_manager_init (GpmManager *manager)
 
 	gpm_debug ("initialising info infrastructure");
 	manager->priv->info = gpm_info_new ();
-	/* logging system needs access to the power stuff... bit of a bodge */
-	gpm_info_set_power (manager->priv->info, manager->priv->power);
 
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
@@ -2568,15 +2666,18 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->last_keyboard_warning = GPM_WARNING_NONE;
 	manager->priv->last_pda_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_warning = GPM_WARNING_NONE;
-	manager->priv->last_primary_percentage_change = 0;
 	manager->priv->done_notify_fully_charged = FALSE;
 
-	/* We don't want to be notified on coldplug if we are on battery power
-	   this should fix #332322 */
+	/* Don't notify on startup if we are on battery power */
 	if (! on_ac) {
 		manager->priv->last_primary_warning = GPM_WARNING_DISCHARGING;
 	}
 
+	/* Don't notify at startup if we are fully charged on AC */
+	if (on_ac) {
+		manager->priv->done_notify_fully_charged = TRUE;
+	}
+
 	/* We can disable this if the ACPI BIOS is fucked, and the
 	   time_remaining is therefore inaccurate or just plain wrong. */
 	use_time = gconf_client_get_bool (manager->priv->gconf_client,
@@ -2628,6 +2729,14 @@ gpm_manager_init (GpmManager *manager)
 	lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
 						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
 	gpm_brightness_set_level_dim (manager->priv->brightness, lcd_dim_brightness);
+
+	/* Do we beep? */
+	manager->priv->enable_beeping = gconf_client_get_bool (manager->priv->gconf_client,
+							       GPM_PREF_ENABLE_BEEPING, NULL);
+
+	/* Do we ignore inhibit requests? */
+	manager->priv->ignore_inhibits = gconf_client_get_bool (manager->priv->gconf_client,
+								GPM_PREF_IGNORE_INHIBITS, NULL);
 }
 
 /**
@@ -2649,9 +2758,13 @@ gpm_manager_finalize (GObject *object)
 	g_return_if_fail (manager->priv != NULL);
 
 	g_object_unref (manager->priv->gconf_client);
+	g_object_unref (manager->priv->hal);
 	g_object_unref (manager->priv->dpms);
 	g_object_unref (manager->priv->idle);
 	g_object_unref (manager->priv->info);
+#ifdef HAVE_POLKIT
+	g_object_unref (manager->priv->polkit);
+#endif
 	g_object_unref (manager->priv->power);
 	g_object_unref (manager->priv->brightness);
 	g_object_unref (manager->priv->tray_icon);
